@@ -8,9 +8,9 @@
     stop("This package requires R 3.5.0 or later")
   if(interactive()) {
     packageStartupMessage(blue(paste("[]==================================================================[]")),appendLF=TRUE)
-    packageStartupMessage(blue(paste("[] Linear Mixed Equations 4 Breeding (lme4breeding) 1.0.7 (2025-08) []",sep="")),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[] Linear Mixed Equations 4 Breeding (lme4breeding) 1.0.8 (2025-12) []",sep="")),appendLF=TRUE)
     packageStartupMessage(paste0(blue("[] Author: Giovanny Covarrubias-Pazaran",paste0(bgGreen(white(" ")), bgWhite(magenta("*")), bgRed(white(" "))),"                        []")),appendLF=TRUE)
-    packageStartupMessage(blue("[] Dedicated to the University of Chapingo and UW-Madison           []"),appendLF=TRUE)
+    packageStartupMessage(blue("[] With the support of the lme4 dev team (Bolker, Bates, et al.)    []"),appendLF=TRUE)
     packageStartupMessage(blue("[] Type 'vignette('lmebreed.gxe')' for a short tutorial             []"),appendLF=TRUE)
     packageStartupMessage(blue("[] Type 'citation('lme4breeding')' to know how to cite it           []"),appendLF=TRUE)
     packageStartupMessage(blue(paste("[]==================================================================[]")),appendLF=TRUE)
@@ -21,7 +21,7 @@
 }
 
 ###
-umat <- function(formula, relmat, data, addmat){
+umat <- function(formula, relmat, data, addmat, k=NULL){
   
   if(missing(data)){stop("Please provide the dataset where we can extract find the factor in formula.")}
   if(missing(relmat)){stop("Please provide the relationship matrix where we will apply the eigen decomposition.")}
@@ -30,50 +30,14 @@ umat <- function(formula, relmat, data, addmat){
   if(!is.list(relmat)){stop("relmat argument should be a named list of relationship matrices", call. = FALSE)}
   
   idProvided <- all.vars(formula)
-  # if(length(idProvided) > 1){stop("Only one factor can be provided in the formula.")}
-  ## build the layout matrix
-  ZrZrt <- list()
-  for(iProv in idProvided){ # iProv = idProvided[1]
-    data$record <- NA
-    if(iProv %in% colnames(data)){
-      ids <- as.character(na.omit(unique(data[,iProv])))
-      for(iId in ids){ # iId<- ids[1]
-        found <- which(data[,iProv] == iId)
-        data[found,"record"] <- 1:length(found)
-      }
-    }else{ # addmat effect
-      if(iProv %in% names(addmat)){
-        if(is.list(addmat[[iProv]])){ # indirect genetic effects
-          ids <- colnames(addmat[[iProv]][[1]])
-          provMat <- Reduce("+", addmat[[iProv]]) # sum matrices
-        }else{ # regular model with single incidence matrix
-          ids <- colnames(addmat[[iProv]])
-          provMat <- addmat[[iProv]]
-        }
-        for(iId in ids){ # iId<- ids[1]
-          found <- which(provMat[,iId] > 0)
-          data[found,"record"] <- 1:length(found)
-        }
-      }else{
-        stop(paste("Your term", iProv, "is neither in the dataset nor in addmat, please correct."), call. = FALSE)
-      }
-    }
-    tabRec <- table(data$record)
-    if(length(tabRec) > 1){
-      if( var(tabRec) > 0 ){stop("The eigen decomposition only works for balanced datasets. Please ensure you fill the dataset to make it balanced for the 'relmat' terms or set to FALSE.", call. = FALSE)}
-    }
-    data$recordF <- as.factor(data$record)
-    nLev <- length(levels(data$recordF))
-    if(nLev > 1){
-      Zr <- sparse.model.matrix(~recordF-1, data=data)
-    }else{
-      Zr <- Matrix::Matrix(1, ncol=1, nrow=nrow(data))
-    }
-    ZrZrt[[iProv]] <- Zr %*% t(Zr)
-  }
+  if(length(idProvided) > 1){message("Only one relationship matrix can be eigen decomposed.")}
+  
   # build the U nxn matrix
-  Ul <- Dl <- Zu <- list()
+  Ul <- Dl <- Zu <- nLev <- list()
+  nLev <- numeric()
   for(iProv in idProvided){
+    nLev[[iProv]] <- max(table(data[,idProvided]))
+    levsInA = unique(data[,idProvided])
     if(iProv %in% colnames(data)){
       Z <- sparse.model.matrix(as.formula(paste("~",iProv,"-1")), data=data)
       colnames(Z) <- gsub(iProv,"", colnames(Z))
@@ -88,28 +52,51 @@ umat <- function(formula, relmat, data, addmat){
         stop(paste("Your term", iProv, "is neither in the dataset nor in addmat, please correct."), call. = FALSE)
       }
     }
-    UD <- eigen(relmat[[iProv]])
+    if(is.null(k)){k<- nrow(relmat[[iProv]])}
+    suppressWarnings( UD <- RSpectra::eigs_sym(relmat[[iProv]], k=k, which = "LM"), classes = "warning")
+    difference <- ncol(relmat[[iProv]]) - k
+    if(difference > 0){
+      UD$values <- c(UD$values, rep(1e-6, difference))
+      UD$vectors <- cbind(UD$vectors, matrix(0, nrow=nrow(UD$vectors), ncol=difference))
+    }
     U<-UD$vectors
     D<-Matrix::Diagonal(x=UD$values)# This will be our new 'relationship-matrix'
     rownames(D) <- colnames(D) <- rownames(relmat[[iProv]])
     rownames(U) <- colnames(U) <- rownames(relmat[[iProv]])
     common <- intersect(colnames(U), colnames(Z))
     Ul[[iProv]]<- U[common,common]
-    # Ul[[iProv]]<- (t(Z[,common]%*%t(U[common,common]))%*%Z[,common])/4
     Dl[[iProv]]<-D[common,common]# This will be our new 'relationship-matrix'
-    Zu[[iProv]] <- Z[,common]
   }
   UnList <- list()
-  for(iel in 1:length(Zu)){
-    UnList[[iel]] <- ( Zu[[iel]] %*% Ul[[iel]] %*% t(Zu[[iel]]) ) * ZrZrt[[iel]]
+  for(iel in 1:1){ # we only use the first relationship matrix for the rotation # length(Ul)
+    UnList[[iel]] <- Matrix::kronecker(Matrix::Diagonal(n=nLev[[iel]]),t(Ul[[iel]]))
   }
-  Utn <- t(Reduce("+",UnList))
-  # ZuBind <- do.call(cbind, Zu)
-  # UBind <- do.call(Matrix::bdiag, Ul)
-  # part1 <- ZuBind%*%UBind%*%t(ZuBind)
-  # W0 <- part0 * part1
-  return(list(Utn=Utn, D=Dl, U=Ul, RRt=ZrZrt, effect=idProvided, record=data$recordF))
+  Utn <- Reduce("+",UnList)
+  
+  if(nrow(Utn) != nrow(data)){
+    stop("The eigen decomposition only works for balanced datasets.\n Please ensure you fill the dataset to make it balanced for the \n 'relmat' terms or set 'rotation' to FALSE.", call. = FALSE)
+  }
+  
+  return(list(Utn=Utn, D=Dl, U=Ul, # RRt=ZrZrt, 
+              effect=idProvided, 
+              record=idProvided # data$recordF
+  ))
 }
+
+balanceData <- function(data, slope=NULL, intercept=NULL){
+  if(is.null(slope)){stop("Please provide the column name corresponsing to the slope (e.g., treatments).", call. = FALSE)}
+  if(is.null(intercept)){stop("Please provide the column name corresponsing to the slope (e.g., treatments).", call. = FALSE)}
+  slopeLevs = unique(data[,slope])
+  data[,paste0(intercept, collapse = "_")] <- apply(data[,intercept],1,function(x){paste0(x, collapse = "_")})
+  interLevs = unique(data[,paste0(intercept, collapse = "_")])
+  balanced = expand.grid(slopeLevs, interLevs)
+  colnames(balanced) <- c(slope,paste0(intercept, collapse = "_"))
+  balanced = merge(balanced, data, by=c(paste0(intercept, collapse = "_"), slope), 
+                   all.x = TRUE)
+  balanced = balanced[ order(balanced[,paste0(intercept, collapse = "_")], balanced[,slope]), ]
+  return(balanced)
+}
+
 ###
 adjBeta <- function(x){
   if(length(x) > 1){
@@ -189,6 +176,7 @@ getMME <- function(object, vc=NULL, recordsToUse=NULL){
     
     ####
     vcov <- do.call(Matrix::bdiag, vc[tn]) # var covar matrix
+    vcov <- vcov + diag(1e-6,ncol(vcov),ncol(vcov))
     LLt <- Matrix::Diagonal( length(unique(object@flist[[iFac]])) ) # digonal for unique number of levels
     rowsi <- list()
     for(j in 1:length(tn)){ # j=1
@@ -463,7 +451,7 @@ imputev <- function (x, method = "median", by=NULL) {
     }
     ms <- aggregate(x~by, FUN=method, na.rm=TRUE)
     rownames(ms) <- ms$by
-    y <- ms[by,"x"]
+    y <- ms[by,2]
     x[which(is.na(x))] <- y[which(is.na(x))]
   } else { # if factor
     tt <- table(x)
@@ -1075,6 +1063,13 @@ A.mat <- function (X, min.MAF = NULL)
   return(A)
 }
 
+I.mat <- function(x){
+  labels0 <- sort(unique(x))
+  II <- Matrix::Diagonal(n=length(labels0))
+  colnames(II) <- rownames(II) <- labels0
+  return(II)
+}
+
 bbasis <- function (x, xl, xr, ndx, deg) 
 {
   tpower <- function(x, t, p) {
